package unicrawler.types;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Page {

	// page URL wrapped in URL obj
	private CrawlerURL mUrl;
	
	// Page content
	private byte[] mPageContent;
	
	// Server response code (200 = OK, 404 = not found etc)
	private int mRespCode;
	
	// how long server took to respond
	private long mRespTime;
	
	// harvested links within the page
	private List<String> mPageLinks;
	
	public Page(CrawlerURL _url, byte[] _content, int _respCode, long _respTime) {
		this.mUrl = _url;
		this.mPageContent = _content;
		this.mRespCode  = _respCode;
		this.mRespTime = _respTime;
		
		this.mPageLinks = new ArrayList<String>();
	}
	
	// very basic implementation atm
	public void parsePageLinks() {
		/*
		 * This is a very crude method / my attempt at normalizing URLs.
		 * Final implementation will include LinkNormalization parser with a less-crude implementation
		 */
		URL normalized = null;
		
		// TODO: remember about charsets!
		String contentStr = new String(this.mPageContent);
		
		// Simple regex detection of href links 
		Pattern hyperlinkPattern = Pattern.compile("(?i)(?s)<\\s*?a.*?href=\"(.*?)\".*?>");
		Matcher hyperlinkMatcher = hyperlinkPattern.matcher(contentStr);
		
		while (hyperlinkMatcher.find()) {
			String match = hyperlinkMatcher.group(1);
			
			// Crude method of dealing with rel hyperlinks
			if (match.startsWith("/")) {
				// relative url
				match = "http://" + this.mUrl.getDomain() + match;
			}
			// Ajax page?
			if (match.startsWith("#")) {
				return;
			}
			
			
			// TODO: Things to think about: email links (mailto), encoded urls (htmentities, '&amp;', '%20%20 ...) etc
			try {
				// this is ugly, but if the url doesn't parse then its not valid...
				normalized = new URL(match);
			}
			catch (MalformedURLException ex) {
				// ignore for now
			}
			
			if (normalized == null) {
				continue;
			}
			
			this.mPageLinks.add(match);
		}
	}
	
	/* GET METHODS */
	public CrawlerURL getPageUrl() {
		return this.mUrl;
	}
	
	public byte[] getPageContent() {
		return this.mPageContent;
	}
	
	public String getPageContentStr() {
		return new String(this.mPageContent);
	}
	
	public int getPageResponseCode() {
		return this.mRespCode;
	}
	
	public long getPageResponseTime() {
		return this.mRespTime;
	}
	
	public List<String> getPageLinks() {
		return this.mPageLinks;
	}
	/* END GET METHODS */
	
	/* SETTER METHODS */
	public void setPageLinks(List<String> _links) {
		this.mPageLinks = _links;
	}
	/* END SETTER METHODS */
}
